# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.service_account_api_key_list_response import ServiceAccountApiKeyListResponse
from ..types.service_account_api_key_response import ServiceAccountApiKeyResponse
from ..types.service_account_list_response import ServiceAccountListResponse
from ..types.service_account_response import ServiceAccountResponse
from .types.service_account_create_account_type import ServiceAccountCreateAccountType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawServiceAccountsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_service_accounts(
        self,
        organization_id: str,
        *,
        page: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ServiceAccountListResponse]:
        """
        List all service accounts in an organization. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        page : typing.Optional[int]
            Page number

        page_size : typing.Optional[int]
            Items per page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ServiceAccountListResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts",
            method="GET",
            params={
                "page": page,
                "page_size": page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountListResponse,
                    parse_obj_as(
                        type_=ServiceAccountListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_service_account(
        self,
        organization_id: str,
        *,
        name: str,
        account_type: ServiceAccountCreateAccountType,
        description: typing.Optional[str] = OMIT,
        assistant_id: typing.Optional[str] = OMIT,
        capabilities: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ServiceAccountResponse]:
        """
        Create a new service account for programmatic API access. Supports 'assistant' type (linked to an assistant) and 'bot' type (standalone). Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        name : str
            Display name for the service account

        account_type : ServiceAccountCreateAccountType
            Account type: 'assistant' or 'bot'

        description : typing.Optional[str]
            Description of what this account does

        assistant_id : typing.Optional[str]
            Assistant ID (required if account_type is 'assistant')

        capabilities : typing.Optional[typing.Sequence[str]]
            List of capability strings to assign to the service account

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ServiceAccountResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts",
            method="POST",
            json={
                "name": name,
                "description": description,
                "account_type": account_type,
                "assistant_id": assistant_id,
                "capabilities": capabilities,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountResponse,
                    parse_obj_as(
                        type_=ServiceAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_service_account(
        self, organization_id: str, service_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ServiceAccountResponse]:
        """
        Get details of a specific service account. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ServiceAccountResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountResponse,
                    parse_obj_as(
                        type_=ServiceAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_service_account(
        self,
        organization_id: str,
        service_account_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        capabilities: typing.Optional[typing.Sequence[str]] = OMIT,
        is_active: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ServiceAccountResponse]:
        """
        Update name, description, capabilities, or active status. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        name : typing.Optional[str]
            Display name

        description : typing.Optional[str]
            Description

        capabilities : typing.Optional[typing.Sequence[str]]
            List of capability strings to assign to the service account

        is_active : typing.Optional[bool]
            Enable or disable the account

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ServiceAccountResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}",
            method="PUT",
            json={
                "name": name,
                "description": description,
                "capabilities": capabilities,
                "is_active": is_active,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountResponse,
                    parse_obj_as(
                        type_=ServiceAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_service_account(
        self, organization_id: str, service_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Any]:
        """
        Soft-delete a service account and revoke all its API keys. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    parse_obj_as(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_service_account_api_keys(
        self, organization_id: str, service_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ServiceAccountApiKeyListResponse]:
        """
        List all API keys belonging to a service account. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ServiceAccountApiKeyListResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}/api-keys",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountApiKeyListResponse,
                    parse_obj_as(
                        type_=ServiceAccountApiKeyListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_service_account_api_key(
        self,
        organization_id: str,
        service_account_id: str,
        *,
        key_name: str,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ServiceAccountApiKeyResponse]:
        """
        Generate a new API key for a service account. The raw key is returned once and must be stored securely. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        key_name : str
            Name for the API key

        expires_at : typing.Optional[dt.datetime]
            Optional expiration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ServiceAccountApiKeyResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}/api-keys",
            method="POST",
            json={
                "key_name": key_name,
                "expires_at": expires_at,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountApiKeyResponse,
                    parse_obj_as(
                        type_=ServiceAccountApiKeyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_service_account_api_key(
        self,
        organization_id: str,
        service_account_id: str,
        key_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Dict[str, typing.Any]]:
        """
        Revoke (soft-delete) an API key belonging to a service account. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        key_id : str
            API key ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Dict[str, typing.Any]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}/api-keys/{jsonable_encoder(key_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Any],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawServiceAccountsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_service_accounts(
        self,
        organization_id: str,
        *,
        page: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ServiceAccountListResponse]:
        """
        List all service accounts in an organization. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        page : typing.Optional[int]
            Page number

        page_size : typing.Optional[int]
            Items per page

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ServiceAccountListResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts",
            method="GET",
            params={
                "page": page,
                "page_size": page_size,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountListResponse,
                    parse_obj_as(
                        type_=ServiceAccountListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_service_account(
        self,
        organization_id: str,
        *,
        name: str,
        account_type: ServiceAccountCreateAccountType,
        description: typing.Optional[str] = OMIT,
        assistant_id: typing.Optional[str] = OMIT,
        capabilities: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ServiceAccountResponse]:
        """
        Create a new service account for programmatic API access. Supports 'assistant' type (linked to an assistant) and 'bot' type (standalone). Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        name : str
            Display name for the service account

        account_type : ServiceAccountCreateAccountType
            Account type: 'assistant' or 'bot'

        description : typing.Optional[str]
            Description of what this account does

        assistant_id : typing.Optional[str]
            Assistant ID (required if account_type is 'assistant')

        capabilities : typing.Optional[typing.Sequence[str]]
            List of capability strings to assign to the service account

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ServiceAccountResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts",
            method="POST",
            json={
                "name": name,
                "description": description,
                "account_type": account_type,
                "assistant_id": assistant_id,
                "capabilities": capabilities,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountResponse,
                    parse_obj_as(
                        type_=ServiceAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_service_account(
        self, organization_id: str, service_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ServiceAccountResponse]:
        """
        Get details of a specific service account. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ServiceAccountResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountResponse,
                    parse_obj_as(
                        type_=ServiceAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_service_account(
        self,
        organization_id: str,
        service_account_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        capabilities: typing.Optional[typing.Sequence[str]] = OMIT,
        is_active: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ServiceAccountResponse]:
        """
        Update name, description, capabilities, or active status. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        name : typing.Optional[str]
            Display name

        description : typing.Optional[str]
            Description

        capabilities : typing.Optional[typing.Sequence[str]]
            List of capability strings to assign to the service account

        is_active : typing.Optional[bool]
            Enable or disable the account

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ServiceAccountResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}",
            method="PUT",
            json={
                "name": name,
                "description": description,
                "capabilities": capabilities,
                "is_active": is_active,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountResponse,
                    parse_obj_as(
                        type_=ServiceAccountResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_service_account(
        self, organization_id: str, service_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Soft-delete a service account and revoke all its API keys. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    parse_obj_as(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_service_account_api_keys(
        self, organization_id: str, service_account_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ServiceAccountApiKeyListResponse]:
        """
        List all API keys belonging to a service account. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ServiceAccountApiKeyListResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}/api-keys",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountApiKeyListResponse,
                    parse_obj_as(
                        type_=ServiceAccountApiKeyListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_service_account_api_key(
        self,
        organization_id: str,
        service_account_id: str,
        *,
        key_name: str,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ServiceAccountApiKeyResponse]:
        """
        Generate a new API key for a service account. The raw key is returned once and must be stored securely. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        key_name : str
            Name for the API key

        expires_at : typing.Optional[dt.datetime]
            Optional expiration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ServiceAccountApiKeyResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}/api-keys",
            method="POST",
            json={
                "key_name": key_name,
                "expires_at": expires_at,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ServiceAccountApiKeyResponse,
                    parse_obj_as(
                        type_=ServiceAccountApiKeyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_service_account_api_key(
        self,
        organization_id: str,
        service_account_id: str,
        key_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Dict[str, typing.Any]]:
        """
        Revoke (soft-delete) an API key belonging to a service account. Requires manage_service_accounts capability.

        Parameters
        ----------
        organization_id : str

        service_account_id : str
            Service account ID

        key_id : str
            API key ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Dict[str, typing.Any]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/organizations/{jsonable_encoder(organization_id)}/service-accounts/{jsonable_encoder(service_account_id)}/api-keys/{jsonable_encoder(key_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Any],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .editor_field_option import EditorFieldOption
from .editor_field_validation import EditorFieldValidation


class EditorField(UniversalBaseModel):
    """
    Single field definition in the editor schema.
    """

    key: str = pydantic.Field()
    """
    Dot-path field key, e.g. 'tool_configurations.system_tools.web_search'
    """

    type: str = pydantic.Field()
    """
    Field type (text, slider, toggle, tool_toggle, select, etc.)
    """

    label: str = pydantic.Field()
    """
    Human-readable label
    """

    hint: typing.Optional[str] = None
    required: typing.Optional[bool] = None
    visible: typing.Optional[bool] = None
    editable: typing.Optional[bool] = None
    default: typing.Optional[typing.Any] = None
    nullable: typing.Optional[bool] = None
    validation: typing.Optional[EditorFieldValidation] = None
    options: typing.Optional[typing.List[EditorFieldOption]] = None
    options_endpoint: typing.Optional[str] = None
    visible_when: typing.Optional[typing.Dict[str, typing.Any]] = None
    create_only: typing.Optional[bool] = None
    edit_only: typing.Optional[bool] = None
    conflicts_with: typing.Optional[str] = None
    exclude_self: typing.Optional[bool] = None
    modes: typing.Optional[typing.List[str]] = None
    is_preview: typing.Optional[bool] = None
    parameters: typing.Optional[typing.List[typing.Dict[str, typing.Any]]] = None
    resource_type: typing.Optional[str] = None
    grouped_by: typing.Optional[str] = None
    read_only: typing.Optional[bool] = None
    data_endpoint: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class StoreUsageEntry(UniversalBaseModel):
    """
    Usage stats for a single store.
    """

    store_id: str = pydantic.Field()
    """
    Vector store identifier
    """

    store_name: str = pydantic.Field()
    """
    Vector store name
    """

    used_bytes: int = pydantic.Field()
    """
    Bytes used by this store
    """

    file_count: int = pydantic.Field()
    """
    Number of files in this store
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AccessOverviewSlice(UniversalBaseModel):
    """
    Slice metadata for access overview (no content).
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the slice
    """

    name: str = pydantic.Field()
    """
    Display name of the slice
    """

    is_composite: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this is a composite slice
    """

    created_by: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the user who created this slice
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from .. import core
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.check_username_response import CheckUsernameResponse
from ..types.device_session_response import DeviceSessionResponse
from ..types.get_users_batch_response import GetUsersBatchResponse
from ..types.image_upload_response import ImageUploadResponse
from ..types.set_current_organization_response import SetCurrentOrganizationResponse
from ..types.update_profile_response import UpdateProfileResponse
from ..types.update_username_response import UpdateUsernameResponse
from ..types.user_profile_response import UserProfileResponse
from ..types.user_response import UserResponse
from ..types.welcome_details_response import WelcomeDetailsResponse
from .raw_client import AsyncRawUserManagementClient, RawUserManagementClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UserManagementClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUserManagementClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUserManagementClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUserManagementClient
        """
        return self._raw_client

    def list_users(
        self,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        active_only: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserResponse]:
        """
        Returns all active users in the system with pagination support.

        Parameters
        ----------
        skip : typing.Optional[int]

        limit : typing.Optional[int]

        active_only : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserResponse]
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.list_users()
        """
        _response = self._raw_client.list_users(
            skip=skip, limit=limit, active_only=active_only, request_options=request_options
        )
        return _response.data

    def create_user(
        self,
        *,
        email: str,
        password: str,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserResponse:
        """
        Creates a new user account. Requires authentication.

        Parameters
        ----------
        email : str

        password : str
            Password (min 8 characters)

        first_name : typing.Optional[str]

        last_name : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.create_user(
            email="email",
            password="password",
        )
        """
        _response = self._raw_client.create_user(
            email=email, password=password, first_name=first_name, last_name=last_name, request_options=request_options
        )
        return _response.data

    def get_current_user_profile(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserProfileResponse:
        """
        Returns the profile of the currently authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserProfileResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.get_current_user_profile()
        """
        _response = self._raw_client.get_current_user_profile(request_options=request_options)
        return _response.data

    def update_user_profile(
        self,
        *,
        username: typing.Optional[str] = OMIT,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        full_name: typing.Optional[str] = OMIT,
        birthday: typing.Optional[dt.date] = OMIT,
        gender: typing.Optional[str] = OMIT,
        profile_image: typing.Optional[str] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        country_id: typing.Optional[str] = OMIT,
        post_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateProfileResponse:
        """
        Updates profile fields for the authenticated user.

        Parameters
        ----------
        username : typing.Optional[str]
            Username

        first_name : typing.Optional[str]
            First name

        last_name : typing.Optional[str]
            Last name

        full_name : typing.Optional[str]
            Full name

        birthday : typing.Optional[dt.date]
            Date of birth

        gender : typing.Optional[str]
            Gender

        profile_image : typing.Optional[str]
            Profile image URL

        timezone : typing.Optional[str]
            Timezone

        country_id : typing.Optional[str]
            Country ID (country_ prefix)

        post_code : typing.Optional[str]
            Postal code

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateProfileResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.update_user_profile()
        """
        _response = self._raw_client.update_user_profile(
            username=username,
            first_name=first_name,
            last_name=last_name,
            full_name=full_name,
            birthday=birthday,
            gender=gender,
            profile_image=profile_image,
            timezone=timezone,
            country_id=country_id,
            post_code=post_code,
            request_options=request_options,
        )
        return _response.data

    def set_current_organization(
        self, *, organization_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> SetCurrentOrganizationResponse:
        """
        Switches the user's active organization context.

        Parameters
        ----------
        organization_id : str
            Organization ID to set as current (org_xxx format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetCurrentOrganizationResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.set_current_organization(
            organization_id="organization_id",
        )
        """
        _response = self._raw_client.set_current_organization(
            organization_id=organization_id, request_options=request_options
        )
        return _response.data

    def get_user_by_email(self, email: str, *, request_options: typing.Optional[RequestOptions] = None) -> UserResponse:
        """
        Looks up a user by their email address. Requires authentication.

        Parameters
        ----------
        email : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.get_user_by_email(
            email="email",
        )
        """
        _response = self._raw_client.get_user_by_email(email, request_options=request_options)
        return _response.data

    def update_username(
        self, *, username: str, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateUsernameResponse:
        """
        Changes the authenticated user's username.

        Parameters
        ----------
        username : str
            New username (3-100 characters)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateUsernameResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.update_username(
            username="username",
        )
        """
        _response = self._raw_client.update_username(username=username, request_options=request_options)
        return _response.data

    def check_username_availability(
        self, *, username: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CheckUsernameResponse:
        """
        Checks if a username is available for use.

        Parameters
        ----------
        username : str
            Username to check

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CheckUsernameResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.check_username_availability(
            username="username",
        )
        """
        _response = self._raw_client.check_username_availability(username=username, request_options=request_options)
        return _response.data

    def get_user_devices(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[DeviceSessionResponse]:
        """
        Returns all active device sessions for the authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DeviceSessionResponse]
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.get_user_devices()
        """
        _response = self._raw_client.get_user_devices(request_options=request_options)
        return _response.data

    def get_welcome_details(self, *, request_options: typing.Optional[RequestOptions] = None) -> WelcomeDetailsResponse:
        """
        Returns onboarding information for the authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WelcomeDetailsResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.get_welcome_details()
        """
        _response = self._raw_client.get_welcome_details(request_options=request_options)
        return _response.data

    def get_user(self, *, request_options: typing.Optional[RequestOptions] = None) -> UserResponse:
        """
        Returns the User model for the currently authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.get_user()
        """
        _response = self._raw_client.get_user(request_options=request_options)
        return _response.data

    def list_deleted_users(
        self,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserResponse]:
        """
        Returns all soft-deleted users. Requires admin access.

        Parameters
        ----------
        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserResponse]
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.list_deleted_users()
        """
        _response = self._raw_client.list_deleted_users(skip=skip, limit=limit, request_options=request_options)
        return _response.data

    def get_user_by_id(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserProfileResponse:
        """
        Returns user details by their user ID.

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserProfileResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.get_user_by_id(
            user_id="user_id",
        )
        """
        _response = self._raw_client.get_user_by_id(user_id, request_options=request_options)
        return _response.data

    def delete_user(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Any]:
        """
        Soft-deletes a user account by ID.

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Any]
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.delete_user(
            user_id="user_id",
        )
        """
        _response = self._raw_client.delete_user(user_id, request_options=request_options)
        return _response.data

    def get_users_batch(
        self, *, user_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> GetUsersBatchResponse:
        """
        Retrieves multiple users by their IDs in a single request.

        Parameters
        ----------
        user_ids : typing.Sequence[str]
            List of user IDs to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUsersBatchResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.get_users_batch(
            user_ids=["user_ids"],
        )
        """
        _response = self._raw_client.get_users_batch(user_ids=user_ids, request_options=request_options)
        return _response.data

    def serve_profile_image(
        self, file_path: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Any:
        """
        Serves a stored profile image. Development only.

        Parameters
        ----------
        file_path : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.serve_profile_image(
            file_path="file_path",
        )
        """
        _response = self._raw_client.serve_profile_image(file_path, request_options=request_options)
        return _response.data

    def upload_profile_image_file(
        self, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> ImageUploadResponse:
        """
        Uploads a profile image from a file.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImageUploadResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.upload_profile_image_file()
        """
        _response = self._raw_client.upload_profile_image_file(file=file, request_options=request_options)
        return _response.data

    def upload_profile_image_url(
        self, *, image_url: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ImageUploadResponse:
        """
        Downloads and sets a profile image from a URL.

        Parameters
        ----------
        image_url : str
            URL of the image to upload

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImageUploadResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.user_management.upload_profile_image_url(
            image_url="image_url",
        )
        """
        _response = self._raw_client.upload_profile_image_url(image_url=image_url, request_options=request_options)
        return _response.data


class AsyncUserManagementClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUserManagementClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUserManagementClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUserManagementClient
        """
        return self._raw_client

    async def list_users(
        self,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        active_only: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserResponse]:
        """
        Returns all active users in the system with pagination support.

        Parameters
        ----------
        skip : typing.Optional[int]

        limit : typing.Optional[int]

        active_only : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.list_users()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_users(
            skip=skip, limit=limit, active_only=active_only, request_options=request_options
        )
        return _response.data

    async def create_user(
        self,
        *,
        email: str,
        password: str,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserResponse:
        """
        Creates a new user account. Requires authentication.

        Parameters
        ----------
        email : str

        password : str
            Password (min 8 characters)

        first_name : typing.Optional[str]

        last_name : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.create_user(
                email="email",
                password="password",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_user(
            email=email, password=password, first_name=first_name, last_name=last_name, request_options=request_options
        )
        return _response.data

    async def get_current_user_profile(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserProfileResponse:
        """
        Returns the profile of the currently authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserProfileResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.get_current_user_profile()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_current_user_profile(request_options=request_options)
        return _response.data

    async def update_user_profile(
        self,
        *,
        username: typing.Optional[str] = OMIT,
        first_name: typing.Optional[str] = OMIT,
        last_name: typing.Optional[str] = OMIT,
        full_name: typing.Optional[str] = OMIT,
        birthday: typing.Optional[dt.date] = OMIT,
        gender: typing.Optional[str] = OMIT,
        profile_image: typing.Optional[str] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        country_id: typing.Optional[str] = OMIT,
        post_code: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateProfileResponse:
        """
        Updates profile fields for the authenticated user.

        Parameters
        ----------
        username : typing.Optional[str]
            Username

        first_name : typing.Optional[str]
            First name

        last_name : typing.Optional[str]
            Last name

        full_name : typing.Optional[str]
            Full name

        birthday : typing.Optional[dt.date]
            Date of birth

        gender : typing.Optional[str]
            Gender

        profile_image : typing.Optional[str]
            Profile image URL

        timezone : typing.Optional[str]
            Timezone

        country_id : typing.Optional[str]
            Country ID (country_ prefix)

        post_code : typing.Optional[str]
            Postal code

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateProfileResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.update_user_profile()


        asyncio.run(main())
        """
        _response = await self._raw_client.update_user_profile(
            username=username,
            first_name=first_name,
            last_name=last_name,
            full_name=full_name,
            birthday=birthday,
            gender=gender,
            profile_image=profile_image,
            timezone=timezone,
            country_id=country_id,
            post_code=post_code,
            request_options=request_options,
        )
        return _response.data

    async def set_current_organization(
        self, *, organization_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> SetCurrentOrganizationResponse:
        """
        Switches the user's active organization context.

        Parameters
        ----------
        organization_id : str
            Organization ID to set as current (org_xxx format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetCurrentOrganizationResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.set_current_organization(
                organization_id="organization_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set_current_organization(
            organization_id=organization_id, request_options=request_options
        )
        return _response.data

    async def get_user_by_email(
        self, email: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserResponse:
        """
        Looks up a user by their email address. Requires authentication.

        Parameters
        ----------
        email : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.get_user_by_email(
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_user_by_email(email, request_options=request_options)
        return _response.data

    async def update_username(
        self, *, username: str, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateUsernameResponse:
        """
        Changes the authenticated user's username.

        Parameters
        ----------
        username : str
            New username (3-100 characters)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateUsernameResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.update_username(
                username="username",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_username(username=username, request_options=request_options)
        return _response.data

    async def check_username_availability(
        self, *, username: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CheckUsernameResponse:
        """
        Checks if a username is available for use.

        Parameters
        ----------
        username : str
            Username to check

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CheckUsernameResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.check_username_availability(
                username="username",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.check_username_availability(
            username=username, request_options=request_options
        )
        return _response.data

    async def get_user_devices(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[DeviceSessionResponse]:
        """
        Returns all active device sessions for the authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[DeviceSessionResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.get_user_devices()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_user_devices(request_options=request_options)
        return _response.data

    async def get_welcome_details(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WelcomeDetailsResponse:
        """
        Returns onboarding information for the authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WelcomeDetailsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.get_welcome_details()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_welcome_details(request_options=request_options)
        return _response.data

    async def get_user(self, *, request_options: typing.Optional[RequestOptions] = None) -> UserResponse:
        """
        Returns the User model for the currently authenticated user.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.get_user()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_user(request_options=request_options)
        return _response.data

    async def list_deleted_users(
        self,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserResponse]:
        """
        Returns all soft-deleted users. Requires admin access.

        Parameters
        ----------
        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.list_deleted_users()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_deleted_users(skip=skip, limit=limit, request_options=request_options)
        return _response.data

    async def get_user_by_id(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserProfileResponse:
        """
        Returns user details by their user ID.

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserProfileResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.get_user_by_id(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_user_by_id(user_id, request_options=request_options)
        return _response.data

    async def delete_user(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.Any]:
        """
        Soft-deletes a user account by ID.

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.Any]
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.delete_user(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_user(user_id, request_options=request_options)
        return _response.data

    async def get_users_batch(
        self, *, user_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> GetUsersBatchResponse:
        """
        Retrieves multiple users by their IDs in a single request.

        Parameters
        ----------
        user_ids : typing.Sequence[str]
            List of user IDs to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUsersBatchResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.get_users_batch(
                user_ids=["user_ids"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_users_batch(user_ids=user_ids, request_options=request_options)
        return _response.data

    async def serve_profile_image(
        self, file_path: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Any:
        """
        Serves a stored profile image. Development only.

        Parameters
        ----------
        file_path : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.serve_profile_image(
                file_path="file_path",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.serve_profile_image(file_path, request_options=request_options)
        return _response.data

    async def upload_profile_image_file(
        self, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> ImageUploadResponse:
        """
        Uploads a profile image from a file.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImageUploadResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.upload_profile_image_file()


        asyncio.run(main())
        """
        _response = await self._raw_client.upload_profile_image_file(file=file, request_options=request_options)
        return _response.data

    async def upload_profile_image_url(
        self, *, image_url: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ImageUploadResponse:
        """
        Downloads and sets a profile image from a URL.

        Parameters
        ----------
        image_url : str
            URL of the image to upload

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ImageUploadResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.user_management.upload_profile_image_url(
                image_url="image_url",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upload_profile_image_url(
            image_url=image_url, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.convert_to_workflow_response import ConvertToWorkflowResponse
from ..types.detach_workflow_response import DetachWorkflowResponse
from ..types.workflow_response import WorkflowResponse
from .raw_client import AsyncRawAssistantWorkflowsClient, RawAssistantWorkflowsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AssistantWorkflowsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAssistantWorkflowsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAssistantWorkflowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAssistantWorkflowsClient
        """
        return self._raw_client

    def convert_assistant_to_workflow(
        self,
        assistant_id: str,
        *,
        workflow_name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConvertToWorkflowResponse:
        """
        Create a workflow from an assistant's settings and link it to the assistant.

        Parameters
        ----------
        assistant_id : str

        workflow_name : typing.Optional[str]
            Workflow name (defaults to '{assistant.name} Workflow')

        description : typing.Optional[str]
            Workflow description

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConvertToWorkflowResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.assistant_workflows.convert_assistant_to_workflow(
            assistant_id="assistant_id",
        )
        """
        _response = self._raw_client.convert_assistant_to_workflow(
            assistant_id, workflow_name=workflow_name, description=description, request_options=request_options
        )
        return _response.data

    def get_assistant_workflow(
        self, assistant_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WorkflowResponse:
        """
        Retrieve the workflow linked to an assistant.

        Parameters
        ----------
        assistant_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkflowResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.assistant_workflows.get_assistant_workflow(
            assistant_id="assistant_id",
        )
        """
        _response = self._raw_client.get_assistant_workflow(assistant_id, request_options=request_options)
        return _response.data

    def detach_assistant_workflow(
        self, assistant_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DetachWorkflowResponse:
        """
        Unlink the workflow from an assistant without deleting the workflow itself.

        Parameters
        ----------
        assistant_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DetachWorkflowResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.assistant_workflows.detach_assistant_workflow(
            assistant_id="assistant_id",
        )
        """
        _response = self._raw_client.detach_assistant_workflow(assistant_id, request_options=request_options)
        return _response.data

    def update_assistant_workflow(
        self,
        assistant_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        definition: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        is_active: typing.Optional[bool] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkflowResponse:
        """
        Update the workflow definition or metadata linked to an assistant.

        Parameters
        ----------
        assistant_id : str

        name : typing.Optional[str]

        description : typing.Optional[str]

        definition : typing.Optional[typing.Dict[str, typing.Any]]
            Updated workflow definition

        is_active : typing.Optional[bool]
            Active status

        external_id : typing.Optional[str]
            Flow-plate internal flow ID for correlation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkflowResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.assistant_workflows.update_assistant_workflow(
            assistant_id="assistant_id",
        )
        """
        _response = self._raw_client.update_assistant_workflow(
            assistant_id,
            name=name,
            description=description,
            definition=definition,
            is_active=is_active,
            external_id=external_id,
            request_options=request_options,
        )
        return _response.data


class AsyncAssistantWorkflowsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAssistantWorkflowsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAssistantWorkflowsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAssistantWorkflowsClient
        """
        return self._raw_client

    async def convert_assistant_to_workflow(
        self,
        assistant_id: str,
        *,
        workflow_name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConvertToWorkflowResponse:
        """
        Create a workflow from an assistant's settings and link it to the assistant.

        Parameters
        ----------
        assistant_id : str

        workflow_name : typing.Optional[str]
            Workflow name (defaults to '{assistant.name} Workflow')

        description : typing.Optional[str]
            Workflow description

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConvertToWorkflowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.assistant_workflows.convert_assistant_to_workflow(
                assistant_id="assistant_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.convert_assistant_to_workflow(
            assistant_id, workflow_name=workflow_name, description=description, request_options=request_options
        )
        return _response.data

    async def get_assistant_workflow(
        self, assistant_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> WorkflowResponse:
        """
        Retrieve the workflow linked to an assistant.

        Parameters
        ----------
        assistant_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkflowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.assistant_workflows.get_assistant_workflow(
                assistant_id="assistant_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_assistant_workflow(assistant_id, request_options=request_options)
        return _response.data

    async def detach_assistant_workflow(
        self, assistant_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DetachWorkflowResponse:
        """
        Unlink the workflow from an assistant without deleting the workflow itself.

        Parameters
        ----------
        assistant_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DetachWorkflowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.assistant_workflows.detach_assistant_workflow(
                assistant_id="assistant_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.detach_assistant_workflow(assistant_id, request_options=request_options)
        return _response.data

    async def update_assistant_workflow(
        self,
        assistant_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        definition: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        is_active: typing.Optional[bool] = OMIT,
        external_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkflowResponse:
        """
        Update the workflow definition or metadata linked to an assistant.

        Parameters
        ----------
        assistant_id : str

        name : typing.Optional[str]

        description : typing.Optional[str]

        definition : typing.Optional[typing.Dict[str, typing.Any]]
            Updated workflow definition

        is_active : typing.Optional[bool]
            Active status

        external_id : typing.Optional[str]
            Flow-plate internal flow ID for correlation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkflowResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.assistant_workflows.update_assistant_workflow(
                assistant_id="assistant_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_assistant_workflow(
            assistant_id,
            name=name,
            description=description,
            definition=definition,
            is_active=is_active,
            external_id=external_id,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.trigger_subscription_list_response import TriggerSubscriptionListResponse
from ..types.trigger_subscription_response import TriggerSubscriptionResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawTriggerSubscriptionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_subscriptions(
        self,
        *,
        organization_id: str,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TriggerSubscriptionListResponse]:
        """
        List all trigger subscriptions for the current user in the given organization.

        Parameters
        ----------
        organization_id : str
            Organization ID

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TriggerSubscriptionListResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/trigger-subscriptions",
            method="GET",
            params={
                "organization_id": organization_id,
                "skip": skip,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionListResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_subscription(
        self,
        *,
        organization_id: str,
        provider: str,
        event_type: str,
        flow_id: str,
        trigger_name: str,
        api_key: str,
        connection_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TriggerSubscriptionResponse]:
        """
        Subscribe a FlowPlate flow to a webhook event. Idempotent: if a subscription with the same (provider, event_type, connection_id, flow_id) already exists for this user, it is returned and re-enabled if disabled.

        Parameters
        ----------
        organization_id : str
            Organization ID

        provider : str
            Provider name (e.g., 'connector', 'freshdesk')

        event_type : str
            Event type (e.g., 'gmail_new_email', 'ticket_created')

        flow_id : str
            FlowPlate flow UUID to trigger

        trigger_name : str
            Trigger node display name in the flow (e.g., 'Answer Tickets')

        api_key : str
            FlowPlate API key for the flow owner (sk-...)

        connection_id : typing.Optional[str]
            Provider-specific account/connection ID. NULL matches all connections.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TriggerSubscriptionResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/trigger-subscriptions",
            method="POST",
            params={
                "organization_id": organization_id,
            },
            json={
                "provider": provider,
                "event_type": event_type,
                "connection_id": connection_id,
                "flow_id": flow_id,
                "trigger_name": trigger_name,
                "api_key": api_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_subscription(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TriggerSubscriptionResponse]:
        """
        Retrieve a single trigger subscription by ID. Only the owner can access it.

        Parameters
        ----------
        subscription_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TriggerSubscriptionResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/trigger-subscriptions/{jsonable_encoder(subscription_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_subscription(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Soft-delete a trigger subscription. It will no longer match incoming webhooks.

        Parameters
        ----------
        subscription_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/trigger-subscriptions/{jsonable_encoder(subscription_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_subscription(
        self,
        subscription_id: str,
        *,
        provider: typing.Optional[str] = OMIT,
        event_type: typing.Optional[str] = OMIT,
        connection_id: typing.Optional[str] = OMIT,
        flow_id: typing.Optional[str] = OMIT,
        trigger_name: typing.Optional[str] = OMIT,
        api_key: typing.Optional[str] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TriggerSubscriptionResponse]:
        """
        Update fields on a trigger subscription. If api_key is provided, it is re-encrypted at rest.

        Parameters
        ----------
        subscription_id : str

        provider : typing.Optional[str]

        event_type : typing.Optional[str]

        connection_id : typing.Optional[str]

        flow_id : typing.Optional[str]

        trigger_name : typing.Optional[str]

        api_key : typing.Optional[str]
            New API key (re-encrypted if provided)

        enabled : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TriggerSubscriptionResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/trigger-subscriptions/{jsonable_encoder(subscription_id)}",
            method="PATCH",
            json={
                "provider": provider,
                "event_type": event_type,
                "connection_id": connection_id,
                "flow_id": flow_id,
                "trigger_name": trigger_name,
                "api_key": api_key,
                "enabled": enabled,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_subscription_s2s(
        self,
        *,
        organization_id: str,
        user_id: str,
        flowplate_master_key: str,
        provider: str,
        event_type: str,
        flow_id: str,
        trigger_name: str,
        api_key: str,
        connection_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TriggerSubscriptionResponse]:
        """
        Called by FlowPlate when a user publishes a flow with trigger components. Authenticated via X-Flowplate-Master-Key. Idempotent: re-enables if the same subscription already exists.

        Parameters
        ----------
        organization_id : str
            Organization ID

        user_id : str
            User ID (flow owner)

        flowplate_master_key : str
            Flowplate master API key

        provider : str
            Provider name (e.g., 'connector', 'freshdesk')

        event_type : str
            Event type (e.g., 'gmail_new_email', 'ticket_created')

        flow_id : str
            FlowPlate flow UUID to trigger

        trigger_name : str
            Trigger node display name in the flow (e.g., 'Answer Tickets')

        api_key : str
            FlowPlate API key for the flow owner (sk-...)

        connection_id : typing.Optional[str]
            Provider-specific account/connection ID. NULL matches all connections.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TriggerSubscriptionResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/trigger-subscriptions/by-flow",
            method="POST",
            params={
                "organization_id": organization_id,
                "user_id": user_id,
            },
            json={
                "provider": provider,
                "event_type": event_type,
                "connection_id": connection_id,
                "flow_id": flow_id,
                "trigger_name": trigger_name,
                "api_key": api_key,
            },
            headers={
                "content-type": "application/json",
                "x-flowplate-master-key": str(flowplate_master_key) if flowplate_master_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_subscriptions_by_flow(
        self,
        flow_id: str,
        *,
        user_id: str,
        flowplate_master_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Dict[str, typing.Any]]:
        """
        Called by FlowPlate when a user unpublishes a flow. Soft-deletes all trigger subscriptions for the given flow_id. Authenticated via X-Flowplate-Master-Key.

        Parameters
        ----------
        flow_id : str

        user_id : str
            User ID (flow owner)

        flowplate_master_key : str
            Flowplate master API key

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Dict[str, typing.Any]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/trigger-subscriptions/by-flow/{jsonable_encoder(flow_id)}",
            method="DELETE",
            params={
                "user_id": user_id,
            },
            headers={
                "x-flowplate-master-key": str(flowplate_master_key) if flowplate_master_key is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Any],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTriggerSubscriptionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_subscriptions(
        self,
        *,
        organization_id: str,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TriggerSubscriptionListResponse]:
        """
        List all trigger subscriptions for the current user in the given organization.

        Parameters
        ----------
        organization_id : str
            Organization ID

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TriggerSubscriptionListResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/trigger-subscriptions",
            method="GET",
            params={
                "organization_id": organization_id,
                "skip": skip,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionListResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_subscription(
        self,
        *,
        organization_id: str,
        provider: str,
        event_type: str,
        flow_id: str,
        trigger_name: str,
        api_key: str,
        connection_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TriggerSubscriptionResponse]:
        """
        Subscribe a FlowPlate flow to a webhook event. Idempotent: if a subscription with the same (provider, event_type, connection_id, flow_id) already exists for this user, it is returned and re-enabled if disabled.

        Parameters
        ----------
        organization_id : str
            Organization ID

        provider : str
            Provider name (e.g., 'connector', 'freshdesk')

        event_type : str
            Event type (e.g., 'gmail_new_email', 'ticket_created')

        flow_id : str
            FlowPlate flow UUID to trigger

        trigger_name : str
            Trigger node display name in the flow (e.g., 'Answer Tickets')

        api_key : str
            FlowPlate API key for the flow owner (sk-...)

        connection_id : typing.Optional[str]
            Provider-specific account/connection ID. NULL matches all connections.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TriggerSubscriptionResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/trigger-subscriptions",
            method="POST",
            params={
                "organization_id": organization_id,
            },
            json={
                "provider": provider,
                "event_type": event_type,
                "connection_id": connection_id,
                "flow_id": flow_id,
                "trigger_name": trigger_name,
                "api_key": api_key,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_subscription(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TriggerSubscriptionResponse]:
        """
        Retrieve a single trigger subscription by ID. Only the owner can access it.

        Parameters
        ----------
        subscription_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TriggerSubscriptionResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/trigger-subscriptions/{jsonable_encoder(subscription_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_subscription(
        self, subscription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Soft-delete a trigger subscription. It will no longer match incoming webhooks.

        Parameters
        ----------
        subscription_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/trigger-subscriptions/{jsonable_encoder(subscription_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_subscription(
        self,
        subscription_id: str,
        *,
        provider: typing.Optional[str] = OMIT,
        event_type: typing.Optional[str] = OMIT,
        connection_id: typing.Optional[str] = OMIT,
        flow_id: typing.Optional[str] = OMIT,
        trigger_name: typing.Optional[str] = OMIT,
        api_key: typing.Optional[str] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TriggerSubscriptionResponse]:
        """
        Update fields on a trigger subscription. If api_key is provided, it is re-encrypted at rest.

        Parameters
        ----------
        subscription_id : str

        provider : typing.Optional[str]

        event_type : typing.Optional[str]

        connection_id : typing.Optional[str]

        flow_id : typing.Optional[str]

        trigger_name : typing.Optional[str]

        api_key : typing.Optional[str]
            New API key (re-encrypted if provided)

        enabled : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TriggerSubscriptionResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/trigger-subscriptions/{jsonable_encoder(subscription_id)}",
            method="PATCH",
            json={
                "provider": provider,
                "event_type": event_type,
                "connection_id": connection_id,
                "flow_id": flow_id,
                "trigger_name": trigger_name,
                "api_key": api_key,
                "enabled": enabled,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_subscription_s2s(
        self,
        *,
        organization_id: str,
        user_id: str,
        flowplate_master_key: str,
        provider: str,
        event_type: str,
        flow_id: str,
        trigger_name: str,
        api_key: str,
        connection_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TriggerSubscriptionResponse]:
        """
        Called by FlowPlate when a user publishes a flow with trigger components. Authenticated via X-Flowplate-Master-Key. Idempotent: re-enables if the same subscription already exists.

        Parameters
        ----------
        organization_id : str
            Organization ID

        user_id : str
            User ID (flow owner)

        flowplate_master_key : str
            Flowplate master API key

        provider : str
            Provider name (e.g., 'connector', 'freshdesk')

        event_type : str
            Event type (e.g., 'gmail_new_email', 'ticket_created')

        flow_id : str
            FlowPlate flow UUID to trigger

        trigger_name : str
            Trigger node display name in the flow (e.g., 'Answer Tickets')

        api_key : str
            FlowPlate API key for the flow owner (sk-...)

        connection_id : typing.Optional[str]
            Provider-specific account/connection ID. NULL matches all connections.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TriggerSubscriptionResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/trigger-subscriptions/by-flow",
            method="POST",
            params={
                "organization_id": organization_id,
                "user_id": user_id,
            },
            json={
                "provider": provider,
                "event_type": event_type,
                "connection_id": connection_id,
                "flow_id": flow_id,
                "trigger_name": trigger_name,
                "api_key": api_key,
            },
            headers={
                "content-type": "application/json",
                "x-flowplate-master-key": str(flowplate_master_key) if flowplate_master_key is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TriggerSubscriptionResponse,
                    parse_obj_as(
                        type_=TriggerSubscriptionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_subscriptions_by_flow(
        self,
        flow_id: str,
        *,
        user_id: str,
        flowplate_master_key: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Dict[str, typing.Any]]:
        """
        Called by FlowPlate when a user unpublishes a flow. Soft-deletes all trigger subscriptions for the given flow_id. Authenticated via X-Flowplate-Master-Key.

        Parameters
        ----------
        flow_id : str

        user_id : str
            User ID (flow owner)

        flowplate_master_key : str
            Flowplate master API key

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Dict[str, typing.Any]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/trigger-subscriptions/by-flow/{jsonable_encoder(flow_id)}",
            method="DELETE",
            params={
                "user_id": user_id,
            },
            headers={
                "x-flowplate-master-key": str(flowplate_master_key) if flowplate_master_key is not None else None,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Dict[str, typing.Any],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.Any],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CategoryStatistics(UniversalBaseModel):
    """
    Schema for category statistics.
    """

    category: str = pydantic.Field()
    """
    Category name
    """

    count: int = pydantic.Field()
    """
    Number of rules in this category
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class KnowledgeVectorStoreResponse(UniversalBaseModel):
    """
    Schema for vector store response in knowledge context.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the vector store
    """

    name: str = pydantic.Field()
    """
    Display name of the vector store
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Description of the vector store
    """

    slice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the primary associated slice
    """

    slice_ids: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    IDs of all slices this vector store is associated with
    """

    organization_id: str = pydantic.Field()
    """
    ID of the organization this vector store belongs to
    """

    created_by: str = pydantic.Field()
    """
    ID of the user who created this vector store
    """

    is_active: bool = pydantic.Field()
    """
    Whether this vector store is currently active
    """

    access_mode: str = pydantic.Field()
    """
    Access control mode: 'organization', 'department', 'private', or 'public'
    """

    file_count: int = pydantic.Field()
    """
    Number of files stored in this vector store
    """

    data_size: int = pydantic.Field()
    """
    Total data size in bytes across all files in the vector store
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Timestamp when the vector store was created
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Timestamp when the vector store was last updated
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.access_grant_list_response import AccessGrantListResponse
from ..types.access_grant_response import AccessGrantResponse
from ..types.access_overview_response import AccessOverviewResponse
from ..types.bulk_grant_entry import BulkGrantEntry
from ..types.resolved_access_response import ResolvedAccessResponse
from ..types.users_with_access_response import UsersWithAccessResponse
from .raw_client import AsyncRawAccessControlClient, RawAccessControlClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AccessControlClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAccessControlClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAccessControlClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAccessControlClient
        """
        return self._raw_client

    def list_grants(
        self,
        organization_id: str,
        *,
        entity_type: typing.Optional[str] = None,
        entity_id: typing.Optional[str] = None,
        grantee_type: typing.Optional[str] = None,
        grantee_id: typing.Optional[str] = None,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantListResponse:
        """
        Lists access grants for an organization with optional filters by entity and grantee. Includes both real grant records and synthesized grants from legacy access modes. Supports pagination and returns enriched grantee names.

        Parameters
        ----------
        organization_id : str

        entity_type : typing.Optional[str]
            Filter by entity type (e.g. 'knowledge_slice', 'vector_store', 'assistant')

        entity_id : typing.Optional[str]
            Filter by specific entity ID

        grantee_type : typing.Optional[str]
            Filter by grantee type (user, department, role, organization)

        grantee_id : typing.Optional[str]
            Filter by specific grantee ID

        skip : typing.Optional[int]
            Number of results to skip for pagination

        limit : typing.Optional[int]
            Maximum number of results to return

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantListResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.list_grants(
            organization_id="organization_id",
        )
        """
        _response = self._raw_client.list_grants(
            organization_id,
            entity_type=entity_type,
            entity_id=entity_id,
            grantee_type=grantee_type,
            grantee_id=grantee_id,
            skip=skip,
            limit=limit,
            request_options=request_options,
        )
        return _response.data

    def create_grant(
        self,
        organization_id: str,
        *,
        entity_type: str,
        entity_id: str,
        grantee_type: str,
        grantee_id: str,
        access_level: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantResponse:
        """
        Creates a new access grant that gives a user, department, role, or organization access to an entity. The grant specifies the entity type, entity ID, grantee, and access level. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        entity_type : str
            Type of entity to grant access to: 'knowledge_slice', 'vector_store', or 'assistant'

        entity_id : str
            ID of the entity to grant access to

        grantee_type : str
            Type of grantee receiving access: 'user', 'department', or 'role'

        grantee_id : str
            ID of the grantee receiving access

        access_level : str
            Access level to grant: 'read', 'write', or 'admin'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.create_grant(
            organization_id="organization_id",
            entity_type="entity_type",
            entity_id="entity_id",
            grantee_type="grantee_type",
            grantee_id="grantee_id",
            access_level="access_level",
        )
        """
        _response = self._raw_client.create_grant(
            organization_id,
            entity_type=entity_type,
            entity_id=entity_id,
            grantee_type=grantee_type,
            grantee_id=grantee_id,
            access_level=access_level,
            request_options=request_options,
        )
        return _response.data

    def get_access_overview(
        self, organization_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AccessOverviewResponse:
        """
        Returns a role-scoped access overview for the Access Overview page. Admins see all slices, stores, and grants. Regular users see only entities they can access. Includes slice-store mappings and composite member relationships.

        Parameters
        ----------
        organization_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessOverviewResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.get_access_overview(
            organization_id="organization_id",
        )
        """
        _response = self._raw_client.get_access_overview(organization_id, request_options=request_options)
        return _response.data

    def list_slice_grants(
        self,
        organization_id: str,
        slice_id: str,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantListResponse:
        """
        Lists all access grants for a specific knowledge slice, including both real grant records and synthesized grants from legacy access modes on the slice's stores. Requires access to the slice.

        Parameters
        ----------
        organization_id : str

        slice_id : str

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantListResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.list_slice_grants(
            organization_id="organization_id",
            slice_id="slice_id",
        )
        """
        _response = self._raw_client.list_slice_grants(
            organization_id, slice_id, skip=skip, limit=limit, request_options=request_options
        )
        return _response.data

    def list_store_grants(
        self,
        organization_id: str,
        store_id: str,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantListResponse:
        """
        Lists all access grants for a specific vector store, including both real grant records and synthesized grants from legacy access modes. Requires access to the store.

        Parameters
        ----------
        organization_id : str

        store_id : str

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantListResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.list_store_grants(
            organization_id="organization_id",
            store_id="store_id",
        )
        """
        _response = self._raw_client.list_store_grants(
            organization_id, store_id, skip=skip, limit=limit, request_options=request_options
        )
        return _response.data

    def list_assistant_grants(
        self,
        organization_id: str,
        assistant_id: str,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantListResponse:
        """
        Lists all access grants for a specific assistant. Returns paginated results with enriched grantee names. Requires organization membership.

        Parameters
        ----------
        organization_id : str

        assistant_id : str

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantListResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.list_assistant_grants(
            organization_id="organization_id",
            assistant_id="assistant_id",
        )
        """
        _response = self._raw_client.list_assistant_grants(
            organization_id, assistant_id, skip=skip, limit=limit, request_options=request_options
        )
        return _response.data

    def get_resolved_access(
        self,
        organization_id: str,
        user_id: str,
        *,
        entity_type: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolvedAccessResponse:
        """
        Computes the effective access for a user across all entity types by resolving direct grants, department grants, role grants, and composite inheritance. Returns separate lists for slice and store access with grant path details.

        Parameters
        ----------
        organization_id : str

        user_id : str

        entity_type : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolvedAccessResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.get_resolved_access(
            organization_id="organization_id",
            user_id="user_id",
        )
        """
        _response = self._raw_client.get_resolved_access(
            organization_id, user_id, entity_type=entity_type, request_options=request_options
        )
        return _response.data

    def get_users_with_access(
        self,
        organization_id: str,
        *,
        entity_type: str,
        entity_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UsersWithAccessResponse:
        """
        Returns all users who have access to a specific entity, including direct grants, department-inherited grants, and role-inherited grants. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        entity_type : str
            Entity type to check access for

        entity_id : str
            Entity ID to check access for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UsersWithAccessResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.get_users_with_access(
            organization_id="organization_id",
            entity_type="entity_type",
            entity_id="entity_id",
        )
        """
        _response = self._raw_client.get_users_with_access(
            organization_id, entity_type=entity_type, entity_id=entity_id, request_options=request_options
        )
        return _response.data

    def bulk_create_grants(
        self,
        organization_id: str,
        *,
        entity_type: str,
        entity_id: str,
        grants: typing.Sequence[BulkGrantEntry],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[AccessGrantResponse]:
        """
        Creates multiple access grants for the same entity in a single request. All grants share the same entity type and entity ID but can have different grantees and access levels. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        entity_type : str
            Type of entity to grant access to: 'knowledge_slice', 'vector_store', or 'assistant'

        entity_id : str
            ID of the entity to grant access to

        grants : typing.Sequence[BulkGrantEntry]
            List of access grants to create in bulk

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[AccessGrantResponse]
            Successful Response

        Examples
        --------
        from aitronos import Aitronos, BulkGrantEntry

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.bulk_create_grants(
            organization_id="organization_id",
            entity_type="entity_type",
            entity_id="entity_id",
            grants=[
                BulkGrantEntry(
                    grantee_type="grantee_type",
                    grantee_id="grantee_id",
                    access_level="access_level",
                )
            ],
        )
        """
        _response = self._raw_client.bulk_create_grants(
            organization_id,
            entity_type=entity_type,
            entity_id=entity_id,
            grants=grants,
            request_options=request_options,
        )
        return _response.data

    def revoke_grant(
        self, organization_id: str, grant_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Soft-deletes an access grant, removing the grantee's access to the entity. The grant record is preserved for audit purposes but no longer enforced. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        grant_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.revoke_grant(
            organization_id="organization_id",
            grant_id="grant_id",
        )
        """
        _response = self._raw_client.revoke_grant(organization_id, grant_id, request_options=request_options)
        return _response.data

    def update_grant(
        self,
        organization_id: str,
        grant_id: str,
        *,
        access_level: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantResponse:
        """
        Updates the access level of an existing access grant. Only the access level can be changed; to change the grantee or entity, revoke and re-create. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        grant_id : str

        access_level : str
            Updated access level: 'read', 'write', or 'admin'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantResponse
            Successful Response

        Examples
        --------
        from aitronos import Aitronos

        client = Aitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )
        client.access_control.update_grant(
            organization_id="organization_id",
            grant_id="grant_id",
            access_level="access_level",
        )
        """
        _response = self._raw_client.update_grant(
            organization_id, grant_id, access_level=access_level, request_options=request_options
        )
        return _response.data


class AsyncAccessControlClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAccessControlClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAccessControlClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAccessControlClient
        """
        return self._raw_client

    async def list_grants(
        self,
        organization_id: str,
        *,
        entity_type: typing.Optional[str] = None,
        entity_id: typing.Optional[str] = None,
        grantee_type: typing.Optional[str] = None,
        grantee_id: typing.Optional[str] = None,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantListResponse:
        """
        Lists access grants for an organization with optional filters by entity and grantee. Includes both real grant records and synthesized grants from legacy access modes. Supports pagination and returns enriched grantee names.

        Parameters
        ----------
        organization_id : str

        entity_type : typing.Optional[str]
            Filter by entity type (e.g. 'knowledge_slice', 'vector_store', 'assistant')

        entity_id : typing.Optional[str]
            Filter by specific entity ID

        grantee_type : typing.Optional[str]
            Filter by grantee type (user, department, role, organization)

        grantee_id : typing.Optional[str]
            Filter by specific grantee ID

        skip : typing.Optional[int]
            Number of results to skip for pagination

        limit : typing.Optional[int]
            Maximum number of results to return

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantListResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.list_grants(
                organization_id="organization_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_grants(
            organization_id,
            entity_type=entity_type,
            entity_id=entity_id,
            grantee_type=grantee_type,
            grantee_id=grantee_id,
            skip=skip,
            limit=limit,
            request_options=request_options,
        )
        return _response.data

    async def create_grant(
        self,
        organization_id: str,
        *,
        entity_type: str,
        entity_id: str,
        grantee_type: str,
        grantee_id: str,
        access_level: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantResponse:
        """
        Creates a new access grant that gives a user, department, role, or organization access to an entity. The grant specifies the entity type, entity ID, grantee, and access level. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        entity_type : str
            Type of entity to grant access to: 'knowledge_slice', 'vector_store', or 'assistant'

        entity_id : str
            ID of the entity to grant access to

        grantee_type : str
            Type of grantee receiving access: 'user', 'department', or 'role'

        grantee_id : str
            ID of the grantee receiving access

        access_level : str
            Access level to grant: 'read', 'write', or 'admin'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.create_grant(
                organization_id="organization_id",
                entity_type="entity_type",
                entity_id="entity_id",
                grantee_type="grantee_type",
                grantee_id="grantee_id",
                access_level="access_level",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_grant(
            organization_id,
            entity_type=entity_type,
            entity_id=entity_id,
            grantee_type=grantee_type,
            grantee_id=grantee_id,
            access_level=access_level,
            request_options=request_options,
        )
        return _response.data

    async def get_access_overview(
        self, organization_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AccessOverviewResponse:
        """
        Returns a role-scoped access overview for the Access Overview page. Admins see all slices, stores, and grants. Regular users see only entities they can access. Includes slice-store mappings and composite member relationships.

        Parameters
        ----------
        organization_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessOverviewResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.get_access_overview(
                organization_id="organization_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_access_overview(organization_id, request_options=request_options)
        return _response.data

    async def list_slice_grants(
        self,
        organization_id: str,
        slice_id: str,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantListResponse:
        """
        Lists all access grants for a specific knowledge slice, including both real grant records and synthesized grants from legacy access modes on the slice's stores. Requires access to the slice.

        Parameters
        ----------
        organization_id : str

        slice_id : str

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantListResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.list_slice_grants(
                organization_id="organization_id",
                slice_id="slice_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_slice_grants(
            organization_id, slice_id, skip=skip, limit=limit, request_options=request_options
        )
        return _response.data

    async def list_store_grants(
        self,
        organization_id: str,
        store_id: str,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantListResponse:
        """
        Lists all access grants for a specific vector store, including both real grant records and synthesized grants from legacy access modes. Requires access to the store.

        Parameters
        ----------
        organization_id : str

        store_id : str

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantListResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.list_store_grants(
                organization_id="organization_id",
                store_id="store_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_store_grants(
            organization_id, store_id, skip=skip, limit=limit, request_options=request_options
        )
        return _response.data

    async def list_assistant_grants(
        self,
        organization_id: str,
        assistant_id: str,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantListResponse:
        """
        Lists all access grants for a specific assistant. Returns paginated results with enriched grantee names. Requires organization membership.

        Parameters
        ----------
        organization_id : str

        assistant_id : str

        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantListResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.list_assistant_grants(
                organization_id="organization_id",
                assistant_id="assistant_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_assistant_grants(
            organization_id, assistant_id, skip=skip, limit=limit, request_options=request_options
        )
        return _response.data

    async def get_resolved_access(
        self,
        organization_id: str,
        user_id: str,
        *,
        entity_type: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ResolvedAccessResponse:
        """
        Computes the effective access for a user across all entity types by resolving direct grants, department grants, role grants, and composite inheritance. Returns separate lists for slice and store access with grant path details.

        Parameters
        ----------
        organization_id : str

        user_id : str

        entity_type : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ResolvedAccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.get_resolved_access(
                organization_id="organization_id",
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_resolved_access(
            organization_id, user_id, entity_type=entity_type, request_options=request_options
        )
        return _response.data

    async def get_users_with_access(
        self,
        organization_id: str,
        *,
        entity_type: str,
        entity_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UsersWithAccessResponse:
        """
        Returns all users who have access to a specific entity, including direct grants, department-inherited grants, and role-inherited grants. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        entity_type : str
            Entity type to check access for

        entity_id : str
            Entity ID to check access for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UsersWithAccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.get_users_with_access(
                organization_id="organization_id",
                entity_type="entity_type",
                entity_id="entity_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_users_with_access(
            organization_id, entity_type=entity_type, entity_id=entity_id, request_options=request_options
        )
        return _response.data

    async def bulk_create_grants(
        self,
        organization_id: str,
        *,
        entity_type: str,
        entity_id: str,
        grants: typing.Sequence[BulkGrantEntry],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[AccessGrantResponse]:
        """
        Creates multiple access grants for the same entity in a single request. All grants share the same entity type and entity ID but can have different grantees and access levels. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        entity_type : str
            Type of entity to grant access to: 'knowledge_slice', 'vector_store', or 'assistant'

        entity_id : str
            ID of the entity to grant access to

        grants : typing.Sequence[BulkGrantEntry]
            List of access grants to create in bulk

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[AccessGrantResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos, BulkGrantEntry

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.bulk_create_grants(
                organization_id="organization_id",
                entity_type="entity_type",
                entity_id="entity_id",
                grants=[
                    BulkGrantEntry(
                        grantee_type="grantee_type",
                        grantee_id="grantee_id",
                        access_level="access_level",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.bulk_create_grants(
            organization_id,
            entity_type=entity_type,
            entity_id=entity_id,
            grants=grants,
            request_options=request_options,
        )
        return _response.data

    async def revoke_grant(
        self, organization_id: str, grant_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Soft-deletes an access grant, removing the grantee's access to the entity. The grant record is preserved for audit purposes but no longer enforced. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        grant_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.revoke_grant(
                organization_id="organization_id",
                grant_id="grant_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.revoke_grant(organization_id, grant_id, request_options=request_options)
        return _response.data

    async def update_grant(
        self,
        organization_id: str,
        grant_id: str,
        *,
        access_level: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AccessGrantResponse:
        """
        Updates the access level of an existing access grant. Only the access level can be changed; to change the grantee or entity, revoke and re-create. Requires the MANAGE_ACCESS_CONTROL capability.

        Parameters
        ----------
        organization_id : str

        grant_id : str

        access_level : str
            Updated access level: 'read', 'write', or 'admin'

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AccessGrantResponse
            Successful Response

        Examples
        --------
        import asyncio

        from aitronos import AsyncAitronos

        client = AsyncAitronos(
            api_key="YOUR_API_KEY",
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.access_control.update_grant(
                organization_id="organization_id",
                grant_id="grant_id",
                access_level="access_level",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_grant(
            organization_id, grant_id, access_level=access_level, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class CreateTriggerSubscriptionRequest(UniversalBaseModel):
    """
    Request body for creating a trigger subscription.
    """

    provider: str = pydantic.Field()
    """
    Provider name (e.g., 'connector', 'freshdesk')
    """

    event_type: str = pydantic.Field()
    """
    Event type (e.g., 'gmail_new_email', 'ticket_created')
    """

    connection_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Provider-specific account/connection ID. NULL matches all connections.
    """

    flow_id: str = pydantic.Field()
    """
    FlowPlate flow UUID to trigger
    """

    trigger_name: str = pydantic.Field()
    """
    Trigger node display name in the flow (e.g., 'Answer Tickets')
    """

    api_key: str = pydantic.Field()
    """
    FlowPlate API key for the flow owner (sk-...)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .user_access_entry_response import UserAccessEntryResponse


class UsersWithAccessResponse(UniversalBaseModel):
    """
    Response for users-with-access endpoint.
    """

    entity_type: str = pydantic.Field()
    """
    Type of entity: 'knowledge_slice', 'vector_store', or 'assistant'
    """

    entity_id: str = pydantic.Field()
    """
    ID of the entity
    """

    users: typing.List[UserAccessEntryResponse] = pydantic.Field()
    """
    List of users with access to this entity
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConfigFieldType = typing.Union[
    typing.Literal[
        "text",
        "integer",
        "number",
        "toggle",
        "date_time",
        "dropdown",
        "multi_select",
        "multi_text",
        "group_select",
        "folder_picker",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UserModelItem(UniversalBaseModel):
    """
    Model usage for a specific user.
    """

    model: str = pydantic.Field()
    """
    Model identifier
    """

    total_cost_chf: typing.Optional[float] = pydantic.Field(default=None)
    """
    Total cost in CHF
    """

    total_requests: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total requests
    """

    total_tokens: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total tokens
    """

    percentage_of_user: typing.Optional[float] = pydantic.Field(default=None)
    """
    Percentage of user's total cost
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class GrantPathEntryResponse(UniversalBaseModel):
    """
    Describes how access was derived.
    """

    type: str = pydantic.Field()
    """
    Type of grant path entry (e.g. 'direct', 'inherited')
    """

    access_level: typing.Optional[str] = pydantic.Field(default=None)
    """
    Access level at this path entry: 'read', 'write', or 'admin'
    """

    department_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Department ID if access was granted via department membership
    """

    role_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Role ID if access was granted via role assignment
    """

    composite_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Composite slice ID if access was inherited from a composite
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
